<?php

/*
 * PHPMoosicLib - A PHP class for easy communication with the moosic jukebox system
 *
 * Copyright (C) 2003 Sven Salzwedel <Sven.Salzwedel@web.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The file xmlrpc.inc.php has another licence, see the source of this File
 * for details.
 */

define('VERSION','0.1');

require_once('xmlrpc.class.php'); // Not GPL!

class PHPMoosic
{
        var $MoosicXmlClient;
        var $ClientConfig;
        
        function connect($host = 'localhost',$port = '30000')
        {
                $this->MoosicXmlClient = new xmlrpc_client('/',$host,$port);
                
                $this->ClientConfig['host'] = $host;
                $this->ClientConfig['port'] = $port;
                
                $result = $this->send_command('api_version');
                return (is_object($result)) ? TRUE : FALSE;
        }
        
        function send_command($command,$values = '')
        {
                $m_message = new xmlrpcmsg($command);
                if (!empty($values))
                {
                        foreach($values as $value)
                        {
                                $m_param = new xmlrpcval($value['val'],$value['type']);
                                $m_message->addParam($m_param);
                                unset($m_param);
                        }
                }
                return $this->MoosicXmlClient->send($m_message);
        }
        
        function moosic_api_version()
        {
                $result            = $this->send_command('api_version');
                $result_val = $result->value();
                $val_array  = $result_val->scalarval();
                $major            = $val_array[0]->scalarval();
                $minor            = $val_array[1]->scalarval();
                
                return array('major' => $major,'minor' => $minor);
        }
        
        function moosic_append($filelist)
        {
                $files = (is_array($filelist)) ? $filelist : array($filelist);
                
                foreach($files as $file)
                { $value[] = new xmlrpcval($file,'base64'); }
                
                $arg[]  = array('val' => $value,'type' => 'array');
                $this->send_command('append',$arg);
        }
        
        function moosic_clear()
        { $this->send_command('clear'); }
        
        function moosic_crop($itemlist)
        {
                $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                
                if (count($items) > 2)
                { return FALSE; }
                
                foreach($items as $item)
                { $value[] = new xmlrpcval($item,'int'); }
                
                $arg[]  = array('val' => $value, 'type' => 'array');
                $this->send_command('crop',$arg);
        }
        
        function moosic_crop_list($itemlist)
        {
                $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                
                foreach($items as $item)
                { $value[] = new xmlrpcval($item,'int'); }
                
                $arg[]  = array('val' => $value, 'type' => 'array');
                $this->send_command('crop_list',$arg);
        }
        
        function moosic_current()
        {
                $result = $this->send_command('current');
                $value  = $result->value();
                
                return $value->scalarval();
        }
        
        function moosic_cut($itemlist)
        {
                $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                
                if (count($items) > 2)
                { return FALSE; }
                
                foreach($items as $item)
                { $value[] = new xmlrpcval($item,'int'); }
                
                $arg[]  = array('val' => $value, 'type' => 'array');
                $this->send_command('cut',$arg);
        }

        function moosic_cut_list($itemlist)
        {
                $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                
                foreach($items as $item)
                { $value[] = new xmlrpcval($item,'int'); }
                
                $arg[]  = array('val' => $value, 'type' => 'array');
                $this->send_command('cut_list',$arg);
        }
        
        function moosic_die()
        { $this->send_command('die'); }
        
        function moosic_filter($regex,$itemlist = '')
        {
                $arg[]  = array('val' => $regex, 'type' => 'base64');
                
                if (!empty($itemlist))
                {
                        if (count($itemlist) > 2)
                        { return FALSE; }
                        
                        foreach($itemlist as $item)
                        { $value[] = new xmlrpcval($item,'int'); }
                        
                        $arg[] = array('val' => $value, 'type' => 'array');
                }
                
                $this->send_command('filter',$arg);
        }
        
        function moosic_get_history_limit()
        {
                $result = $this->send_command('get_history_limit');
                $value  = $result->value();
                
                return $value->scalarval();
        }
        
        function moosic_halt_queue()
        { $this->send_command('halt_queue'); }
        
        function moosic_haltqueue()
        { $this->send_command('haltqueue'); }
        
        function moosic_history($limit = '')
        {
                if (!empty($limit))
                { $arg[] = array('val' => $limit, 'type' => 'int'); }

                $result         = $this->send_command('history',$arg);
                $result_val = $result->value();
                $val_array  = $result_val->scalarval();
                
                $count = 0;
                foreach($val_array as $hist_item)
                {
                        $item = $hist_item->scalarval();
                        $history[$count]['file']        = $item[0]->scalarval();
                        $history[$count]['start'] = (int) $item[1]->scalarval();
                        $history[$count]['end']   = (int) $item[2]->scalarval();
                        unset($item,$hist_item);
                        $count++;
                }
                return $history;
        }
        
        function moosic_indexed_list($itemlist = '')
        {
                if (!empty($itemlist))
                {
                        $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                        
                        if (count($items) > 2)
                        { return FALSE; }
                        
                        foreach($items as $item)
                        { $value[] = new xmlrpcval($item,'int'); }
                        
                        $arg[] = array('val' => $value, 'type' => 'array');
                }
                
                $result     = $this->send_command('indexed_list',$arg);
                $result_val = $result->value();
                $val_array  = $result_val->scalarval();

                $indexed_list['start'] = $val_array['start']->scalarval();
                $list = $val_array['list']->scalarval();

                foreach($list as $item)
                { $indexed_list['list'][] = $item->scalarval();        }
                
                return $indexed_list;
        }
        
        function moosic_insert($filelist,$pos)
        {
                $files = (is_array($filelist)) ? $filelist : array($filelist);
                
                foreach($files as $file)
                { $value[] = new xmlrpcval($file,'base64'); }
                
                $arg[] = array('val' => $value, 'type' => 'array');
                $arg[] = array('val' => $pos, 'type' => 'int');
                
                $this->send_command('insert',$arg);
        }
        
        function moosic_is_looping()
        {
                $result = $this->send_command('is_looping');
                $value  = $result->value();
                
                return ($value->scalarval() == 1) ? TRUE : FALSE;
        }
        
        function moosic_is_paused()
        {
                $result = $this->send_command('is_paused');
                $value  = $result->value();
                
                return ($value->scalarval() == 1) ? TRUE : FALSE;
        }

        function moosic_is_queue_running()
        {
                $result = $this->send_command('is_queue_running');
                $value  = $result->value();
                
                return ($value->scalarval() == 1) ? TRUE : FALSE;
        }
        
        function moosic_last_queue_update()
        {
                $result = $this->send_command('last_queue_update');
                $value  = $result->value();
                
                return (int) $value->scalarval();
        }
        
        function moosic_length()
        {
                $result = $this->send_command('length');
                $value  = $result->value();
                
                return $value->scalarval();
        }
        
        function moosic_list($itemlist = '')
        {
                if (!empty($itemlist))
                {
                        $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                        
                        if (count($items) > 2)
                        { return FALSE; }
                        
                        foreach($items as $item)
                        { $value[] = new xmlrpcval($item,'int'); }
                        
                        $arg[] = array('val' => $value, 'type' => 'array');
                }
                
                $result     = $this->send_command('list',$arg);
                $result_val = $result->value();
                $val_array  = $result_val->scalarval();
                
                foreach($val_array as $file)
                { $list[] = $file->scalarval(); }
                
                return $list;
        }
        
        function moosic_move($itemlist,$pos)
        {
                $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                
                if (count($items) > 2)
                { return FALSE; }
                
                foreach($items as $item)
                { $value[] = new xmlrpcval($item,'int'); }
                
                $arg[] = array('val' => $value, 'type' => 'array');
                $arg[] = array('val' => $pos, 'type' => 'int');
                
                $this->send_command('move',$arg);
        }
        
        function moosic_move_list($itemlist,$pos)
        {
                $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                
                foreach($items as $item)
                { $value[] = new xmlrpcval($item,'int'); }
                
                $arg[] = array('val' => $value,'type' => 'array');
                $arg[] = array('val' => $pos, 'type' => 'int');
                
                $this->send_command('move_list',$arg);
        }
        
        function moosic_next()
        { $this->send_command('next'); }
        
        function moosic_pause()
        { $this->send_command('pause'); }
        
        function moosic_prepend($filelist)
        {
                $files = (is_array($filelist)) ? $filelist : array($filelist);
                
                foreach($files as $file)
                { $value[] = new xmlrpcval($file,'base64'); }
                
                $arg[] = array('val' => $value, 'type' => 'array');
                
                $this->send_command('prepend',$arg);
        }
        
        function moosic_putback()
        { $this->send_command('putback'); }
        
        function moosic_queue_length()
        {
                $result = $this->send_command('queue_length');
                $value  = $result->value();
                
                return $value->scalarval();
        }
        
        function moosic_reconfigure()
        { $this->send_command('reconfigure'); }
        
        function moosic_remove($regex,$itemlist = '')
        {
                $arg[] = array('val' => $regex,'type' => 'base64');
                
                if (!empty($itemlist))
                {
                        $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                        
                        if (count($items) > 2)
                        { return FALSE; }
                        
                        foreach($items as $item)
                        { $value[] = new xmlrpcval($item,'int'); }
                        
                        $arg[] = array('val' => $value, 'type' => 'array');
                }
                
                $this->send_command('remove',$arg);
        }
        
        function moosic_replace($filelist)
        {
                $files = (is_array($filelist)) ? $filelist : array($filelist);
                
                foreach($files as $file)
                { $value[] = new xmlrpcval($file,'base64'); }
                
                $arg[] = array('val' => $value, 'type' => 'array');
                
                $this->send_command('replace',$arg);
        }

        function moosic_reverse($itemlist = '')
        {
                if (!empty($itemlist))
                {
                        $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                        if (count($items) > 2)
                        { return FALSE; }

                        foreach($items as $item)
                        { $value[] = new xmlrpcval($item,'int'); }

                        $arg[] = array('val' => $value, 'type' => 'array');
                }

                $this->send_command('reverse',$arg);
        }
        
        function moosic_run_queue()
        { $this->send_command('run_queue'); }

        function moosic_runqueue()
        { $this->send_command('runqueue'); }

        function moosic_set_history_limit($limit)
        {
                $arg[] = array('val' => $limit, 'type' => 'int');

                $this->send_command('set_history_limit',$arg);
        }

        function moosic_set_loop_mode($mode)
        {
                $arg[] = array('val' => $mode, 'type' => 'boolean');

                $this->send_command('set_loop_mode',$arg);
        }

        function moosic_showconfig()
        {
                $result = $this->send_command('showconfig');
                $value  = $result->value();

                return $value->scalarval();
        }

        function moosic_shuffle($itemlist = '')
        {
                if(!empty($itemlist))
                {
                        $items = (is_array($itemlist)) ? $itemlist : array($itemlist);
                        
                        if (count($items) > 2)
                        { return FALSE; }

                        foreach($items as $item)
                        { $value[] = new xmlrpcval($item, 'int'); }

                        $arg[] = array('val' => $value, 'type' => 'array');
                }

                $this->send_command('shuffle', $arg);
        }

        function moosic_sort($itemlist = '')
        {
                if(!empty($itemlist))
                {
                        $items = (is_array($itemlist)) ? $itemlist : array($itemlist);

                        if(count($items) > 2)
                        { return FALSE; }

                        foreach($items as $item)
                        { $value[] = new xmlrpcval($item, 'int'); }

                        $arg[] = array('val' => $value, 'type' => 'array');
                }

                $this->send_command('sort',$arg);
        }

        function moosic_toggle_loop_mode()
        { $this->send_command('toggle_loop_mode'); }

        function moosic_toggle_pause()
        { $this->send_command('toggle_pause'); }

        function moosic_unpause()
        { $this->send_command('unpause'); }

        function moosic_version()
        {
                $result = $this->send_command('version');
                $value  = $result->value();

                return $value->scalarval();
        }
}
?>
